/*
				DIWRJPEG.C
				
This file contains code to write a JPEG image from a bitmap
source . The function to call from Visual Basic is DIWriteJpg().
See the "README1.txt" file for insructions on how to use this
library from Visual Basic .

*/

#define DI_FILE_IO

#include "DICommon.h"	/*Common decls for DILIB*/
#include "cdjpeg.h"		/* Common decls for cjpeg/djpeg applications */



/*Original IJG code*/
#define JMESSAGE(code,string)	string ,

static const char * const cdjpeg_message_table[] = {
#include "cderror.h"
  NULL
};


LOCAL(cjpeg_source_ptr)/*MODIFIED--we know Src is .bmp*/
select_file_type (j_compress_ptr cinfo, FILE * infile)
{
    return jinit_read_bmp(cinfo);
}
static char * outfilename;	/* for -outfile switch */
/*End of original IJG code*/





/*--------------------------------------------------------------*\
  Function : DIWRITEJPG											
			 Writes a JPEG file from a Bitmap source according  
			 to the specified parameters .

  Parameters:	outfile_path : full path to destination file
				quality		 : image quality (1-100%)
				progressive  : (0 or 1) write progressive JPEG
				grayscale	 : (0 or 1) write grayscale JPEG

  Return values : See DICommon.h
  Note : source Bitmap must be "C:\tmp.bmp" 

\*--------------------------------------------------------------*/
long DIAPI DIWriteJpg ( char *DestPath		,
						long quality        ,
						long progressive    )

{
	struct jpeg_compress_struct cinfo;
	struct jpeg_error_mgr jerr;
	
	cjpeg_source_ptr src_mgr;
	FILE * input_file;
	FILE * output_file;
	JDIMENSION num_scanlines;

	cinfo.err = jpeg_std_error(&jerr);
	jpeg_create_compress(&cinfo);

	jerr.addon_message_table = cdjpeg_message_table;
	jerr.first_addon_message = JMSG_FIRSTADDONCODE;
	jerr.last_addon_message = JMSG_LASTADDONCODE;

	cinfo.in_color_space = JCS_RGB;
	jpeg_set_defaults(&cinfo);
	
	if(progressive==1)
		#ifdef C_PROGRESSIVE_SUPPORTED
			jpeg_simple_progression(&cinfo);
		#endif


	jpeg_quality_scaling(quality);
	jpeg_set_quality(&cinfo, quality, FALSE);


	outfilename = DestPath; 

	if (outfilename == NULL)
		return DI_ERR_OUTFILE;
	
	
	/* Open the input file. */
	if ((input_file = fopen("C:\\tmp.bmp", READ_BINARY)) == NULL) 
		return DI_ERR_INFILE;

	/* Open the output file. */
	if ((output_file = fopen(outfilename, WRITE_BINARY)) == NULL) 
		return DI_ERR_OUTFILE;


	/* Figure out the input file format, and set up to read it. */
	src_mgr = select_file_type(&cinfo, input_file);
	src_mgr->input_file = input_file;

	/* Read the input file header to obtain file size & colorspace. */
	(*src_mgr->start_input) (&cinfo, src_mgr);

	/* Now that we know input colorspace, fix colorspace-dependent defaults */
	jpeg_default_colorspace(&cinfo);

	/* Adjust default compression parameters by re-parsing the options */
	//file_index = parse_switches(&cinfo, argc, argv, 0, TRUE);

	/* Specify data destination for compression */
	jpeg_stdio_dest(&cinfo, output_file);

	/* Start compressor */
	jpeg_start_compress(&cinfo, TRUE);

	/* Process data */
	while (cinfo.next_scanline < cinfo.image_height) {
		num_scanlines = (*src_mgr->get_pixel_rows) (&cinfo, src_mgr);
		(void) jpeg_write_scanlines(&cinfo, src_mgr->buffer, num_scanlines);
	}

	/* Finish compression and release memory */
	(*src_mgr->finish_input) (&cinfo, src_mgr);
	jpeg_finish_compress(&cinfo);
	jpeg_destroy_compress(&cinfo);

	/* Close files, if we opened them */
	if (input_file != stdin)
		fclose(input_file);
	if (output_file != stdout)
		fclose(output_file);

	//return value
	return ( DI_SUCCESS ) ;
}
